#KIP-Modell mit einer anderen  Zielfunktion
#Analyse Darstellung zwischen [Min.Schulen ; Max. Schuhlen] (NEUE SCHULEN!)
#
# usage: perl kip_eConstraint.pl [-x lambda0] [-y lambda1] [-z lambda2] [-k lambda3] [-t lambda4] [-k 0.  [-l $maxSchulenMaxSchueler] [-u $maxSchulenMaxSchulen]
#
# output: output/kip_eConstraint.txt
#
# temporary: output/temporary/kip_eConstraint-tmp.txt

#Initialisieren der Default-Werte
$lambda0 = 1;									#Gewichtungsfaktor fuer erste Zielfunktion (Schueler)
$lambda1 = 0.00001;								#Gewichtungsfaktor fuer zweite Zielfunktion (neue Schulen)
$lambda2 = 0.000001;								#Gewichtungsfaktor fuer dritte Zielfunktion (Schulen)
$lambda3 = 0.000001;								#Gewichtungsfaktor fuer dritte Zielfunktion (Klassen)
$lambda4 = 0.00001;								#Gewichtungsfaktor fuer dritte Zielfunktion (Termine)

$minS = 1;									    #minimale Anzahl an neuen Schulen
$maxS = 100;									#maximale Anzahl an neuen Schulen

use Getopt::Std;								#Uebergabeparameter parsen
$myoptString = "'x:y:z:k:t:l:u:'";
%options = ();
getopts("$myoptString", \%options);
$lambda0 = $options{x} if defined $options{x};
$lambda1 = $options{y} if defined $options{y};
$lambda2 = $options{z} if defined $options{z};
$lambda3 = $options{k} if defined $options{k};
$lambda4 = $options{t} if defined $options{t};
$minS = $options{l} if defined $options{l};     # Anzahl Schulen bei Schuelermaximierung
$maxS = $options{u} if defined $options{u};     # Anzahl Schulen bei Schulmaximierung

$ZIMPL = "";
$SCIP = "";
if($^O eq 'MSWin32'){

    $ZIMPL = "Start /B /W win_binary/zimpl.exe";
    $SCIP = "Start /B /W win_binary/scip.exe";

}
elsif($^O eq 'linux'){
    
    $ZIMPL = "zimpl";
    $SCIP = "scip";
    
}
else {

    die "Unknown operating system!\n";
    
}

$filenameLog="output/log.txt";

unlink "output/kip_eConstraint.txt";		#Loeschen von Daten von eventuellen vorherigen Berechnungen

open(OUTFILE, ">>output/kip_eConstraint.txt") or die "unable to open output/kip_eConstraint.txt: $!\n";
$Datum = localtime(time);
print OUTFILE "generated: $Datum\n\n";
print OUTFILE "Comparison between different quantities of new Schools and their impact on other values:\n\n";
close(OUTFILE);

#==========================================================================
#Berechnung des Modells fuer die jeweilige Anzahl an Schulen
#==========================================================================

for ($s = $minS; $s <= $maxS; $s += 1) {	

	print "$s ";
	open (LOG, ">>$filenameLog") or die "unable to open $filenameLog: $!\n";
	print LOG "solving model for nl = $s\n";
	
	#Erstellung des ZIMPL-Models
    print LOG "perl scripts/zimplmodel.pl $lambda0 $lambda1 $lambda2 $lambda3 $lambda4 kip_eConstraint $s\n";
    close(LOG);
    
	system("perl scripts/zimplmodel.pl $lambda0 $lambda1 $lambda2 $lambda3 $lambda4 kip_eConstraint $s 1>> $filenameLog") == 0 or die "an error occured with scripts/zimplmodel.pl: $!\n";
	
	#Ausfuehrung von ZIMPL
	open (LOG, ">>$filenameLog") or die "unable to open $filenameLog: $!\n";
	print LOG "$ZIMPL -o output/zimpl/kip_eConstraint output/zimpl/kip_eConstraint.zpl\n";
    close(LOG);
    
	system ("$ZIMPL -o output/zimpl/kip_eConstraint output/zimpl/kip_eConstraint.zpl 1>> $filenameLog") == 0 or die "failed to create lp file with ZIMPL: $!\n";

	#Ausfuehrung von SCIP
	open (LOG, ">>$filenameLog") or die "unable to open $filenameLog: $!\n";
	print LOG "$SCIP -b scripts/scip/kip_eC.bat\n";
    close(LOG);
    
	system("$SCIP -b scripts/scip/kip_eC.bat 1>> $filenameLog") == 0 or die "an error occured with SCIP: $!\n";

	#==========================================================================
	#Ausgabe der Loesung
	#==========================================================================
	@schul_ids = ();
	@schulen = ();
	%K_zu_S = ();
	$total_Anz_S = 0;
	$i = 0;
	$y = 0;
	$r = 0;
	$ue = 0;
	%anm1 = ();

	open(OUTFILE, ">>output/kip_eConstraint.txt") or die "unable to open output/kip_eConstraint2.txt: $!\n";
	$oldfilehandle = select(OUTFILE);

#format OUTFILE = 
#@<<<@<<<@<<<@<<<<<@<<<
#@aktuelle_lsg
#.

	open(FILE, 'output/temporary/kip_eConstraint-tmp.txt') or die "unable to open output/temporary/kip_eConstraint2-tmp.txt: $!\n";
#	@aktuelle_lsg = ($ywert, $Schulen, $Anzahl, $total_Anz_S, $belegte_Termine);

	while (<FILE>) {
		$t = $_;
		@ov = split(" ", $t);
		if($ov[2] > 0) { 
			$value = $ov[2];	
		}
		elsif($i > 1){
			$anmeldungen[$i] = $_;					#Die eingelesene Zeile von der Lösungsausgabe
			@aktuelle_zeile = split("#", $anmeldungen[$i]);		#Split der eingelesenen Zeile nach #
			@aktuelle_zeile2 = split(" ", $aktuelle_zeile[0]);
			$bla = $aktuelle_zeile2[0];
	        if($aktuelle_zeile[0] eq "x"){
				$klassen_id = $aktuelle_zeile[1];		#Auslesen der klassen_id
				if($aktuelle_zeile[2] =~ /([0-9]*) */){		#Auslesen der termin_id
					$Termin_id = $1;
				}
			    $anm1{$klassen_id} = "$Termin_id";
				$y++;
			}
			if($aktuelle_zeile[0] eq "z"){
				$r++;
			}
			if($aktuelle_zeile[0] eq "u"){
				$ue++;
			}
			if($aktuelle_zeile2[0] eq "yneu"){
				$ywert = $aktuelle_zeile2[1];
			}
			if($aktuelle_zeile2[0] eq "ysum"){
				$Schulen = $aktuelle_zeile2[1];
			}
		}		
		$i++;
	}
	close(FILE);

	%anm = ();
	@kl_liste;
	while (($klassen_id, $Termin_id) = each %anm1) {
		if (!exists($anm{$Termin_id})){					#Test, ob Schluessel termin_id existiert, dann Hinzufuegen der Klasse zu Termin
			@kl_liste = ($klassen_id);
			@anm{$Termin_id} = "@kl_liste";
		}
		else{
			@kl_liste = @anm{$Termin_id};
			push(@kl_liste, $klassen_id);				#sonst push zu bereits vorhandener Klassen
			@anm{$Termin_id} = join(",", @kl_liste);
		}
	}
	@end_termine = sort keys %anm;	
	$belegte_Termine = @end_termine;	 				#Anzahl belegter Termine 

	open(OUTFILE2, 'input/main.txt') or die "unable to open input/main.txt: $!\n";
	while (<OUTFILE2>){
		chomp();
		($u, $b, $c, $d, $e) = split(/\t/, $_);
		if($u eq "k"){
			$k_id_s{$b} = "$c";	    				#Schule einer Klasse
		} 
		if($u eq "a"){
			$klassenGr{($c, $d)} = "$e";	 			#Groesse  von (Klasse,Termin) = eine 'Anmeldung'
		} 
	}
	close(OUTFILE2);

	foreach $f (sort{$a<=>$b} keys %anm1){	
		$s_von_k{$f} = "($k_id_s{$f})";
		$total_Anz_S += $klassenGr{($f, $anm1{$f})}; 		 	#Anzahl Teilnehmer
	}

	while (($g, $oo) = each %s_von_k){					#beteiligte Schulen --> mit Liste von Klassen
		if (!exists($K_zu_S{$oo})){			
			@Liste = ($g);
			@K_zu_S{$oo} = "@Liste";
		}
		else{
			@Liste = @K_zu_S{$oo};
			push(@Liste, $g);	
			@K_zu_S{$oo} = join(", ", @Liste);
		}
	}	

# was passiert hier?
#	$Schulen = $i - $y - 2;							#Alle Einträge in der Loesung - Klassen - Text nich bei L in Zielfunktion!
	foreach (sort{$a<=>$b} keys %K_zu_S){
		#print(" $_\n");
	}
	
	foreach $w (keys %K_zu_S){	
		push(@Bezirke, $w);
	}

	$Anzahl_Bezirke = @Bezirke;
	$Anzahl = $y;								#y= eine Anmeldung

#	@aktuelle_lsg = ($ywert, $Schulen, $Anzahl , $total_Anz_S, $belegte_Termine);

#	write;
	print OUTFILE "$ywert new schools ($Schulen schools total) with $total_Anz_S pupils, $Anzahl classes and $belegte_Termine dates\n";

	select($oldfilehandle);
	close(OUTFILE);

	undef %anm;
	undef %anm1;
	undef %K_zu_S;
	undef %s_von_k;
}
print "\n";
