# Hauptprogrammung fuer die normale Optimierung 
#
# usage: perl kip_main.pl [-x lambda0] [-y lambda1] [-z lambda2] [-k lambda3] [-t lambda4] [-m minNeueSchulen] [-n minSchulenAlle] [-o minSchueler] [-p minKlassen] [-q minTermine]
#
# input:  input/main.txt: KIP-Datendatei (vom Server)
#         input/exceptions.txt: MultipleClasses-Termine (manuell)
#         input/regions.txt: Bezirksnamen (statisch)
#         input/conflicts.txt: Klassenkonflikte (Bearbeitung)
#         input/doMatch.txt: gesetzte Klassen (Bearbeitung)
#         input/static.txt: gesetzte Anmeldungen (Bearbeitung)
#
# output: output/zimpl/kip_main.zpl
#         output/zimpl/kip_main.lp
#         output/kip_main.txt
#         output/kip_main-detail.txt: Statistikfile
#         output/kip_main-db.txt: Datenbankupload

# Initialisieren der Default-Werte
$lambda0 = 1;									#Gewichtungsfaktor fuer erste Zielfunktion (Schueler)
$lambda1 = 0.00001;								#Gewichtungsfaktor fuer zweite Zielfunktion (neue Schulen)
$lambda2 = 0.000001;								#Gewichtungsfaktor fuer dritte Zielfunktion (Schulen)
$lambda3 = 0.000001;								#Gewichtungsfaktor fuer dritte Zielfunktion (Klassen)
$lambda4 = 0.00001;								#Gewichtungsfaktor fuer dritte Zielfunktion (Termine)


use Getopt::Std;								#Uebergabeparameter parsen
$myoptString = "'x:y:z:k:t:m:n:o:p:q:'";
%options = ();
getopts("$myoptString", \%options);
$lambda0 = $options{x} if defined $options{x};
$lambda1 = $options{y} if defined $options{y};
$lambda2 = $options{z} if defined $options{z};
$lambda3 = $options{k} if defined $options{k};
$lambda4 = $options{t} if defined $options{t};
$minSchulen = 0 if defined $options{n}; # Parameter muss gesetzt sein
$minSchulen = $options{m} if defined $options{m};  #Uebergabe-Parameter fuer mindest-Anzahl an neuen Schulen
$minSchulenAlle = $options{n} if defined $options{n};  #Uebergabe-Parameter fuer mindest-Anzahl an Schulen insgesamt
$minSchueler = $options{o} if defined $options{o};
$minKlassen = $options{p} if defined $options{p};
$minTermine = $options{q} if defined $options{q};

$ZIMPL = "";
$SCIP = "";
if($^O eq 'MSWin32'){

    $ZIMPL = "Start /B /W win_binary/zimpl.exe";
    $SCIP = "Start /B /W win_binary/scip.exe";

}
elsif($^O eq 'linux'){
    
    $ZIMPL = "zimpl";
    $SCIP = "scip";
    
}
else {

    die "Unknown operating system!\n";
    
}

#==========================================================================
# Durchfuehrung der Berechnungen
#==========================================================================

# Loeschen alter Dateien
unlink "output/zimpl/kip_main.zpl";
unlink "output/zimpl/kip_main.lp";
print "\n" . "-"x76 . "\nremoved old files output/zimpl/kip_main.[zpl,lp]...\n";

# Erstellung des ZIMPL-Modells mit zimplmodel.pl
print "\ngenerate ZPL model:\n";
print "perl scripts/zimplmodel.pl $lambda0 $lambda1 $lambda2 $lambda3 $lambda4 kip_main $minSchulen $minSchulenAlle $minSchueler $minKlassen $minTermine\n";
system("perl scripts/zimplmodel.pl $lambda0 $lambda1 $lambda2 $lambda3 $lambda4 kip_main $minSchulen $minSchulenAlle $minSchueler $minKlassen $minTermine") 
    == 0 or die "an error has occured with scripts/zimplmodel.pl: $!\n";

# Erstellung des IP-Modells mit zimpl
print "generate IP model:\n";
print "$ZIMPL -o output/zimpl/kip_main output/zimpl/kip_main.zpl\n";
system("$ZIMPL -o output/zimpl/kip_main output/zimpl/kip_main.zpl") == 0 or die "failed to create lp file with ZIMPL: $!\n";

# Loseung des IP-Modells mit scip
print "solve IP model:\n";
print "$SCIP -b scripts/scip/kip_main.bat\n";
system("$SCIP -b scripts/scip/kip_main.bat")  == 0 or die "an error occured with SCIP: $!\n";

#==========================================================================
# Erstellung der Ausgabe der Loesungen
#==========================================================================

@schul_ids = ();
@schulen = ();
@term_with_appl = ();
$sum_kap = 0;
$netto_sch = 0;
%K_zu_S = ();
$total_Anz_Schueler = 0;
my @loesungszeile = ();
$value;
#use strict;

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#Einlesen der Eingabedaten der Anmeldungen
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

open(OUTFILE2, 'input/main.txt') or die "$0: unable to open input/main.txt: $!\n";
	while (<OUTFILE2>) {
	chomp();
	($u, $b, $c, $d, $e, $ff, $gg) = split(/\t/,$_);
		if( $u eq "k" ){
			$k_id_s{$b} = "$c";					#"%k_id_s":  klassen_id --> Schule_id
			$k_id_b{$b} = "$ff";					#"%k_id_b":  klassen_id--> Bezirk_id
			$k_id_n{$b} = "$d";					#"%k_id_n":  klassen_id--> Name
			$anzahl_k++;						#Zaehler fuer Anzahl der angemeldeten Klassen
		} 
		if( $u eq "a" ){
			$klassenGr{($c,$d)} = "$e";				#Klassengroesse fuer eine eine 'Anmeldung' von (Klasse,Termin)
			$klassenGro{($c)} = "$e";				#"%klassenGro":  anmeldung_id --> Groesse der Klasse
			$kl_id_von_anm{($b)} = "$c";				#"%kl_id_von_anm": Anmeldung_id --> klassen_id
			$a_id{($c,$d)} = "$b"; 					#definiere "%a_id": (klassen_id, schule_id) -->  Anmeldung _id
			$g_von_a{$b} = "$e";					#Anmeldumg_id --> Termin_id's
			$t_zu_a{$b} = "$d";
			push(@term_with_appl,$d);			#Alle Termine, die eine Anmeldung erhielten, in einem Array
			$anzahl_a++;						#Zaehler fuer Anzahl der Anmeldungen
		}	
# Da die Schulliste mit s nicht weiter genutzt wurde, wurde sie getilgt ($zaehler_schulen ist immer 0)
#		if( $u eq "s" ){
#			$zaehler_schulen++;					#Zaehler fuer Anzahl der angemeldeten Schulen
#		} 

		if( $u eq "t" ){
			$kap{$b} = "$d";  					#kap: termin_id --> Kapazitae	t eines Termins
			$sum_kap = $sum_kap + $d;				#Gesamtkapazitaet	
			$zaehler_termine++;	
			push(@alleTermine,$b);
			$kurs_zu_t{$b} = "$c";

			$kursname{$b} = "$gg";				#termin_id --> Kursbezeichnung
			
		} 
	}
close(OUTFILE2);

open(OUTFILE4, 'input/capIncrease.txt') or die "$0: unable to open input/capIncrease.txt: $!\n";			# beachte kaperhoehung
	while (<OUTFILE4>) {
	chomp();
	($u2, $b2, $c2) = split(/\t/,$_);
	if( $u2 eq "t" ){		  					
			$sum_kap = $sum_kap + $c2;				#Gesamtkapazitaet	
		} 
}
close(OUTFILE4);
 
#Pro Klasse --> Eine Liste der Anmeldungen %A_zu_K
while (($a1,$k1) = each %kl_id_von_anm) {					#"%A_zu_K": klassen_id --> Liste der Anmeldungen ("@Liste5")
	if (! exists($A_zu_K{$k1})) {						#Testet ob die Klasse "$k1"  noch nicht betrachtet wurde
		my @Liste5 = ($a1);						#Hinzufuegen erster Anmeldung "$a1" zu Klasse "$k1"				
		@A_zu_K{$k1} = "@Liste5";
	}
	else{									#Ansonsten: Hinzufuegen zu bereits vorhandener Klasse
		my @Liste5 = @A_zu_K{$k1};					
		push(@Liste5, $a1);
		@A_zu_K{$k1} = join(",", @Liste5);
	}
}

foreach my $h (sort{$a<=>$b} keys %A_zu_K){					#Fuer alle angemeldeten klassen_id's aus "%A_zu_K" :
	$netto_sch += $klassenGro{($h)};					#Zaehle Anzahl Schueler = Gesamt Anzahl Schueler
}

#Pro angemeldete Schule --> Eine Liste der angemeldeten Klassen %S_zu_K
while (($v, $w) = each %k_id_s) {						#"S_zu_K": Angemeldete Schulen --> Klassen Liste
	if (! exists($S_zu_K{$w})) {						#Testet, ob die Schule "$w"  noch nicht betrachtet wurde
		@Liste2 = ($v);
		@S_zu_K{$w} = "@Liste2";					#Hinzufuegen erster Klasse "$v" zu Schule "$w"
	}
	else{
		@Liste2 = @S_zu_K{$w};						#Ansonsten: Hinzufuegen zu bereits vorhandener Schule
		push(@Liste2, $v);
		@S_zu_K{$w} = join(",", @Liste2);
	}
}

@schools = sort keys %S_zu_K;							#Alle angemeldeten Schhul_id's
$angemeldete_schulen = @schools;						#Anzahl angemeldeter Schulen

#Pro angemeldete Klasse--> Eine Liste der Bezike %B_zu_K
while (($q, $l) = each %k_id_b) {						#"B_zu_K":  Angemeldete Bezirke --> Klassen Liste
	if (! exists($B_zu_K{$l})) {						#Testet, ob Bezirk "$l", noch nicht betrachtet wurde
		@Liste3 = ($q);							#Hinzufuegen erster Klasse "$q" zu Bezirk "$l"
		@B_zu_K{$l} = "@Liste3";
	}
	else{
		@Liste3 = @B_zu_K{$l};						#Ansonsten: Hinzufuegen zu bereits vorhandenem Bezirk
		push(@Liste3, $q);
		@B_zu_K{$l} = join(",", @Liste3);
	}
}

@end_bezirke = sort keys %B_zu_K;						#Alle angemeldeten Bezirk_id's
$angemeldete_bezirke = @end_bezirke;						#Anzahl angemeldeter Bezirke

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Einlesen der SCIP-Loesung
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

open(FILE, 'output/kip_main.txt') or die "can't open output/kip_main.txt: $!\n";

@aktuelle_lsg = ($schulen, $total_klassen, $total_Anz_Schueler, $belegte_Termine);

$yneu = 0;
$ysum = 0;
$yL3Y = 0;
$yL3Ytotal = 0;
@yL3YNTTT;
$l3=0;
while (<FILE>) {
	my $t = $_;
#	print "t: ", $t;
	my @ov = split(" ", $t);
	if($ov[2] > 0) { 
		$value = $ov[2];
		}
	elsif($i > 1){
		$loesungszeile[$i] = $_;					#Die eingelesene Zeile von der SCIP-Ausgabe der Zuordnung
		my @aktuelle_zeile = split("#", $loesungszeile[$i]);		#Splitten der eingelesenen Zeile nach
		my @aktuelle_zeile2 = split(" ", $aktuelle_zeile[0] );
		if($aktuelle_zeile2[0] eq "yneu"){
			$yneu = $aktuelle_zeile2[1]; 
#			print "yneu: <$yneu> az <@aktuelle_zeile> az2 <@aktuelle_zeile2>";
		}
		if($aktuelle_zeile2[0] eq "yL3Y"){			
			$yL3Y = $aktuelle_zeile2[1]; 
		}
		if($aktuelle_zeile2[0] eq "yL3Ytotal"){			
			$yL3Ytotal = $aktuelle_zeile2[1]; 
		}
		if($aktuelle_zeile2[0] eq "yL3YNTTT"){			
			@yL3YNTTT[$l3] = split(" ", $aktuelle_zeile[1] );
			$l3++;
		}
		if($aktuelle_zeile2[0] eq "ysum"){
			$ysum = $aktuelle_zeile2[1];
#			print "ysum: <$ysum> az <@aktuelle_zeile> az2 <@aktuelle_zeile2>";
		}
		if($aktuelle_zeile[0] eq "x"){
			my $klassen_id = $aktuelle_zeile[1];			#Einlesen der klassen_id
			if($aktuelle_zeile[2] =~ /([0-9]*) */){ 		#Einlesen der termin_id
				$termin_id = $1;
			}		
			$anmeldungen{$klassen_id} = "$termin_id";  
			$anm_zaehler++;						#Zaehler der zugeordneten Anmeldugen (Klassen)
		}
	}		
	$i++;
}
close(FILE);

#open(FILE, 'output/kip_main.txt') or 
#	die "Datei kip_main.txt kann nicht geoeffnet werden: $!\n";

#============================================================================
# Ausgabedatei erstellen							# in kip_main-detail
#============================================================================

 open(OUTFILE, ">output/kip_main-detail.txt") or die "can't open output/kip_main-detail.txt: $!\n";
$oldfilehandle=select(OUTFILE);

$Datum = localtime(time);
print OUTFILE "generated: $Datum\n\n";

#Format der Ausgabedatei
format OUTFILE =
@<<<@<<<@<<<<@<<
@aktuelle_lsg
.

#Interpretation der Loesungdsaten aus "kip_main.txt"
%anm = ();
@kl_liste;									#Liste der Klassen pro Anmeldung
while (($klassen_id, $termin_id) = each %anmeldungen) {
	if (! exists($anm{$termin_id})) {					#Testet ob Termin mit termin_id noch nicht betrachtet wurde
		@kl_liste = ($klassen_id);					#Hinzufuegen erster Klasse "$klassen_id" zu Termin "termin_id"
		@anm{$termin_id} = "@kl_liste";					
	}
	else{
		@kl_liste = @anm{$termin_id};					#Ansonsten: Hinzufuegen der Klasse zu Klassenliste
		push(@kl_liste, $klassen_id);
		@anm{$termin_id} = join(",", @kl_liste);
	}
}

@end_termine = sort keys %anm;							#Alle termin_id's 1x mit jeweiligen Liste der Klassen
$belegte_Termine = @end_termine;	 					#Anzahl belegter Termine

print "dates that were not assigned: date ID, course ID\n";
foreach $term (@alleTermine) {							#Bestimmung der nicht zu geordneten Termine
	$drin = 0;
	foreach $term2 (@end_termine) {
		if ($term == $term2){
			$drin = 1;
			break;
		}
	}
	if($drin == 0){
		print "$term, $kurs_zu_t{$term} ($kursname{$term})";
		$gotAppl = 0;
		foreach $term3 (@term_with_appl) {
			if ($term == $term3){
				$gotAppl = 1;
				break;
			}
		}
		if($gotAppl == 0){
			print "\t"x3,"[No application for this date!]";
		}
		print "\n";
	}
}

print "*"x76,"\n";

foreach $klassen_id2 (sort{$a<=>$b} keys %anmeldungen) {			#Bestimmung aller Bezirke aus denen Schulen angemeldet wurden
	if (! exists($bez_zu_k{$k_id_b{$klassen_id2}})) {
		@liste7 = ($klassen_id2);
		@bez_zu_k{$k_id_b{$klassen_id2}} = "@liste7";
	}
	else{
		@liste7 = @bez_zu_k{$k_id_b{$klassen_id2}};
		push(@liste7, $klassen_id2);
		@bez_zu_k{$k_id_b{$klassen_id2}} = join(",", @liste7);
	}
}

@end_bezirk = sort keys %bez_zu_k;
$total_bezirke = @end_bezirk;	

#Detail Ausgabe: Termin-Kurs-Klasse-Bezirk
print "assignments in detail:\n","-"x76,"\n";
foreach $term(sort{$a<=>$b} keys %anm) {
	print("date ID: $term (course $kurs_zu_t{$term}: $kursname{$term})\n");
	@klassen_des_terms = split(",", @anm{$term});
	foreach $klas (@klassen_des_terms){					#Fuer alle Termine wird die Information ausgegeben welche Klasse
		print("class: $klas, $k_id_n{$klas}, $k_id_b{$klas}\n");	#aus welchem Bezirk zugeordnet wurde
	}
	print "-"x76 . "\n";
}

foreach $f (sort{$a<=>$b} keys %anmeldungen){
	$s_von_k{$f} = "$k_id_s{$f}";
	$total_Anz_Schueler += $klassenGr{($f, $anmeldungen{$f})};		#Anzahl zugeordneter Schueler
}

#Pro beteiligte Schule--> Eine Liste der beteiligten Klassen %K_zu_S
while ( (my $k, my $s) = each %s_von_k) {					#"%K_zu_S": beteiligte Schulen_id --> Liste der Klassen
	if (! exists($K_zu_S{$s})) {						#Testet, ob Schule "$s" noch nicht betrachtet wurde
		my @Liste = ($k);
		@K_zu_S{$s} = "@Liste";						#Hinzufuegen erster Klasse "$k" zu Schule "$s"
	}
	else{
		my @Liste = @K_zu_S{$s};					#Ansonsten: Hinzufuegen der Klasse zu bereits vorhanderer Schule
		push(@Liste, $k);
		@K_zu_S{$s} = join(",", @Liste);
	}
}

$schulen = keys %K_zu_S;							#Anzahl zugeordneter Schhulen
$total_klassen = $anm_zaehler;

#============================================================================
#Loesungsausgabe (Statistik)							# auch in kip_main-detail
#============================================================================

print "*********************\n\n";
print "amount of applications:  $anzahl_a\n";
print "amount of registered classes:  $anzahl_k\n";
print "amount of registered schools:  $angemeldete_schulen\n";
print "amount of registered regions:  $angemeldete_bezirke\n";
print "total capacity:  $sum_kap\n";

#print "Realisierte Zuordnung 2009 (Schulen, Klassen, Schueler, belegte Termine):\n";
#@aktuelle_lsg = ($schulen, $total_klassen , $total_Anz_Schueler, $belegte_Termine);
#write;

$prozT = ($total_Anz_Schueler/$sum_kap) * 100;					#Nutzbare Kapazitaet (d.h. Kapazitaet - SGAP)
print "\n";
print "assigned capacity: $total_Anz_Schueler of $sum_kap = $prozT %\n";

$prozS = ($total_Anz_Schueler/$netto_sch) * 100;
print "assigned pupils: $total_Anz_Schueler of $netto_sch = $prozS %\n";

$Anteil_klassen = ($total_klassen/$anzahl_k) * 100;
$prozSchule = ($schulen/$angemeldete_schulen) * 100;
$prozSL3Y = 100;
if ($yL3Ytotal > 0){
  $prozSL3Y = ($yL3Y/$yL3Ytotal) * 100;
}
print "assigned classes: $total_klassen of $anzahl_k = $Anteil_klassen %\n";
print "assigned schools: $schulen of $angemeldete_schulen = $prozSchule %\n";
print "assigned schools that applied the last two years but weren't selected then: $yL3Y of those $yL3Ytotal schools got assigned this time ($prozSL3Y %)\n";
$prozBezirk = ($total_bezirke/$angemeldete_bezirke) * 100;
print "assigned regions: $total_bezirke of $angemeldete_bezirke = $prozBezirk %\n";

$prozTermine = ($belegte_Termine/$zaehler_termine) * 100;
print "assigned dates: $belegte_Termine of $zaehler_termine = $prozTermine %\n";

#Checks
# siehe Zeile 110
#if ($zaehler_schulen != $angemeldete_schulen){
#	print "\n";
#	print "Anzahl Schulen pruefen!\n";
#}

if ($zaehler_termine < $belegte_Termine){ 
	print "Check amount of dates!";
}
if ($zaehler_termine > $belegte_Termine){
	$leer_termin = $zaehler_termine - $belegte_Termine;
	print "amount of dates without an assignment: $leer_termin!";
}
print "\n";

if ($total_bezirke = $angemeldete_bezirke){
	print "\n";
	print "All registered regions were assigned!\n";
}
#else{
#	print "amount of assigned regions = $total_bezirke!\n"; #redundante Information
#}

print "\n";
print "amount of new schools that were assigned = $yneu\n";
print "amount of assigned schools total = $ysum\n";
print "\n------------------------\n\n";
	
#Testet, ob eine Klasse sich mehrmals mit untercshiedlicher Groesse angemeldet hat!
print "applications of classes with varying sizes:\n";
$found_v_size = 0;
foreach $h2 (sort{$a<=>$b} keys %A_zu_K) {					#"%A_zu_K": Pro Klasse --> Liste der Anmeldungen 
	@anm_der_klasse = split(",", @A_zu_K{$h2});				#@anm_der_klasse = Menge der Anmeldungen in je die Listen
	$anm_in_liste = @anm_der_klasse;					#$anm_in_liste Anzahl der Anmeldungen pro Klasse $h2
	foreach $a3 (@anm_der_klasse){						#Fuer alle Anmeldugen a3 einer Klasse => Summe der Schueler
		$netto_sch2 += $g_von_a{$a3};	
	}
	$netto_sch2=$netto_sch2/$anm_in_liste;					#Summe der Schueler(der Klasse h2) =
	if ($netto_sch2!=$klassenGro{$h2}){			
		@anm_der_klasse=split(",",@A_zu_K{$h2});
		foreach $a3 (@anm_der_klasse){	
			print "Check size of class $h2 --> $a3: $g_von_a{$a3} for date $t_zu_a{$a3} (course $kurs_zu_t{$t_zu_a{$a3}})!\n";
		}
		$found_v_size = 1;
	}
	$netto_sch2 = 0;
	$anm_in_liste = 0;
}
if($found_v_size == 0) {
	print "No class registered with varying sizes!\n";
}
print "\n";									#Ausgabe der Klassen mit grosser Anmeldungsgroesse
print "applications with more than 30 pupils:\n";
#Testet, ob eine Klasse sich mit mehr als 30 Schuelern angemeldet hat.
$found_more30 = 0;
foreach $h2 (sort{$a<=>$b} keys %A_zu_K) {					#"%A_zu_K": Pro Klasse-->Liste der Anmeldungen 
	@anm_der_klasse = split(",", @A_zu_K{$h2});				#@anm_der_klasse =Menge der Anmeldungen in je die Listen
	#print"Klasse $h2,-->hat $klassenGro{$h2} Schueler \n";			#= Gesamt Summe der Schueler aller Anm. der Klasse h2/ Anzahl Anmeldungen der Klasse
	if($klassenGro{$h2} > 30){
		foreach $a3 (@anm_der_klasse){
			if($g_von_a{$a3} > 30){
				print "size of class $h2 --> $a3: $g_von_a{$a3} for date $t_zu_a{$a3} (course $kurs_zu_t{$t_zu_a{$a3}}) is larger than 30!\n";
			}
		}
		$found_more30 = 1;
	}
}
if($found_more30 == 0) {
	print "No class registered with more than 30 pupils!\n";
}

#print  "\nAnalyse zu Schulen, die sich die letzten beiden Jahre vergeblich angemeldet haben und nun wieder nicht genommen wurden: \n";
#print  "@yL3YNTTT \n";
select($oldfilehandle);
close(OUTFILE);

print "statistics have been written to file output/kip_main-detail.txt\n";

#============================================================================
# Erstellung der Datei zum Upload in die Datenbank
#============================================================================

open(OUTFILE3, ">output/kip_main-db.txt") or die "unable to open output/kip_main-db.txt: $!\n";
$oldfilehandle = select(OUTFILE3);

#insert into tablemath values (@<<<<<, @<<<<<, @<<<<<);

format OUTFILE3 =
@<<<<< @<<<<< @<<<<<
@aktuelle_lsg
.

foreach my $K (sort{$a<=>$b} keys %anmeldungen) {				#Fuer alle Klassen K n der Loesung:
	my $T = $anmeldungen{$K};						#liefert den ensprechenden zugeordneten Termin
	my $A = $a_id{($K,$T)};							#liefert die Anmeldung_id
	@aktuelle_lsg = ($A,$K,$T);						#Fuege die konkrete Zuordnung
	
	write;
}

select($oldfilehandle);
close(OUTFILE3);

print "data base upload has been written to file output/kip_main-db.txt\n\n";
