# Programm zur Erzeugung des zimpl-Files
#
# usage:  perl scripts/zimplmodel.pl lambda0 lambda1 lambda2 lambda3 lambda4 kip_main minNeueSchulen minSchulen
#         perl scripts/zimplmodel.pl lambda0 lambda1 lambda2 lambda3 lambda4 kip_gapAnalyzation Termin-Id
#         perl scripts/zimplmodel.pl lambda0 lambda1 lambda2 lambda3 lambda4 kip_eConstraint nNeueSchulen
#         perl scripts/zimplmodel.pl lambda0 lambda1 lambda2 lambda3 lambda4 kip_eConstraint2 nSchulen
#         perl scripts/zimplmodel.pl lambda0 lambda1 lambda2 lambda3 lambda4 kip_capacityIncrement Inc Termin-Id [nSchulen (neu)]
#
# input:  input/main.txt: KIP-Datendatei (vom Server)
#         input/restriktionen.txt: SingleClass-Kurse (manuell)
#         input/regions.txt: Bezirksnamen (statisch)
#         input/conflicts.txt: Klassenkonflikte (Bearbeitung)
#         input/doMatch.txt: gesetzte Klassen (Bearbeitung)
#         input/static.txt: gesetzte Anmeldungen (Bearbeitung)
#
# output: output/zimpl/model.zpl (model = $ARGV[5])
#
# lambda0: Zielfunktionsgewichtung Anzahl Schueler
# lambda1: Zielfunktionsgewichtung Anzahl neue Schulen
# lambda2: Zielfunktionsgewichtung Anzahl Schulen
# lambda3: Zielfunktionsgewichtung Anzahl Klassen
# lambda4: Zielfunktionsgewichtung Anzahl Termine
# kip_*:  Modellauswahl
#         kip_main: Schuelermaximierung
#         kip_gapAnalyzation: Single-Knapsack-Gap
#         kip_eConstraint: Schueler vs. Schulen
#         kip_capacityIncrement: Kapazitätsanalyse
# minNeueSchulen: Mindestanzahl von Schulen (optional)
# minSchulen: Mindestanzahl von Schulen (optional, geht nur mit minNeueSchulen)
# nSchulen: Anzahl von Schulen
# Inc: Kapazitaetsincrement

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# KIP Zimpl-Modell
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#------------------------------------------------------------------------------
# Template: Formatstring mit Platzhaltern %s zur Erzeugung von Varianten mit print
#------------------------------------------------------------------------------

my $TEMPLATE = << 'END_OF_TEMPLATE';

	#KIP-Modell
	#KinderUni --- Multiple-Knapsack-Problem
	#------------------------------------------
    #String1: Zielfunktionsgewichtung Anzahl Schueler
    #String2: Zielfunktionsgewichtung Anzahl neue Schulen
    #String3: Zielfunktionsgewichtung Anzahl Schulen
    #String4: Zielfunktionsgewichtung Anzahl Klassen 
    #String5: Zielfunktionsgewichtung Anzahl Termine 
    #String6: Parameter (optional)
    #String7: zusaetzliche Variablen (optional)
    #String7b: zusaetzlicher Zielfunktionsterm
    #String8: forallJ
    #String9: zusaetzliche Bedingungen (optional)

	param DAT  := "input/main.txt";                            	# KIP-Datendatei
	param OBJX := %s;                                           	# String1: Zielfunktionsgewichtung Anzahl Schueler
	param OBJY := %s;                                           	# String2: Zielfunktionsgewichtung Anzahl neue Schulen
	param OBJZ := %s;                                           	# String3: Zielfunktionsgewichtung Anzahl Schulen
	param OBJK := %s;					    	# String4: Zielfunktionsgewichtung Anzahl Klassen	
	param OBJT := %s;					    	# String5: Zielfunktionsgewichtung Anzahl Termine
	param OBJ3Y := 10000;					    	# Paramater fuer Zielfunktion, Schulen die in den letzten 3 Jahren nicht genommen wurden obwohl angemeldet 

	param AGE  := 1;                                            # unzulaessige Altersdifferenz

	%s                                                          # String6: Parameter
	
    # Anmeldungsliste
	set Applications := {read DAT as "<3n, 4n>" match "^a"};				# Anmeldungen: Klassen-Ids, Termin-Ids
	param a[Applications] := read DAT as "<3n, 4n> 5n" match "^a"; 		# Klassengroesse einer Anmeldung
	set AppliedClasses := proj(Applications,<1>);									    # Klassen-Ids

    # Terminliste
	set Dates := {read DAT as "<2n>" match "^t"};					# Termin-Ids (evtl. ohne Anmeldung)
	param b[Dates] := read DAT as "<2n> 4n" match "^t";				# Kapazitaet der Termine
	param cmin[Dates] := read DAT as "<2n> 5n" match "^t";			# minimale Klassenstufe der Termine
	param cmax[Dates] := read DAT as "<2n> 6n" match "^t";			# maximale Klassenstufe der Termine

    # Klassenliste
    # k	klassen_id	schul_nr	klassenname	klassenstufe	bezirk
	set Classes := {read DAT as "<2n, 3n, 6s>" match "^k"};			# Klassen_id Schule Bezirk
#	set ClassIDs := {read DAT as "<2n>" match "^k"};                   # Nochmal die Klassen explizit gelistet
	set ClassIDs := proj(Classes,<1>);                                       # Auflistung aller Klassen
	set Schools := proj(Classes,<2>);									    # Schul-Ids
	set Regions := proj(Classes,<3>);									    # Bezirke
	param ClassGrade[ClassIDs] := read DAT as "<2n> 5n" match "^k";			# Klassenstufe der Klasse
	param SchoolOfClass[AppliedClasses] := read DAT as "<2n> 3n" match "^k";			# Schule der Klasse
	param RegionOfClass[AppliedClasses] := read DAT as "<2n> 6s" match "^k";			# Bezirk der Klasse

    # Schulliste (wird nicht verwendet)

    # Beteiligungsliste
	set Beteiligte := {read DAT as "<2n, 4n, 5n, 6n, 7n, 8n, 9n, 10n, 11n, 12n, 13n>" match "^v"};	
	set Smin5temp := {<i, j, k, l, m, n, o ,p ,q, r, s> in Beteiligte with j > 0};		   	 # Teilmenge Schulen beteiligt vor 5 Jahren
	set Smin4temp := {<i, j, k, l, m, n, o ,p ,q, r, s> in Beteiligte with k > 0};		   	 # Teilmenge Schulen beteiligt vor 4 Jahren
	set Smin3temp := {<i, j, k, l, m, n, o ,p ,q, r, s> in Beteiligte with l > 0};		   	 # Teilmenge Schulen beteiligt vor 3 Jahren
	set Smin2temp := {<i, j, k, l, m, n, o ,p ,q, r, s> in Beteiligte with m > 0};		   	 # Teilmenge Schulen beteiligt vor 2 Jahren
 	set Smin1temp := {<i, j, k, l, m, n, o ,p ,q, r, s> in Beteiligte with n > 0};		   	 # Teilmenge Schulen beteiligt im Vorjahr
	set Smin5 := proj(Smin5temp, <1>);
	set Smin4 := proj(Smin4temp, <1>);
	set Smin3 := proj(Smin3temp, <1>);
	set Smin2 := proj(Smin2temp, <1>);
	set Smin1 := proj(Smin1temp, <1>);
	set Sold := Smin5 union Smin4 union Smin3 union Smin2 union Smin1;
	set Sold2years := Smin1 union Smin2;
	set Snew := Schools - Sold2years;										# Ids aller Schulen, die noch nie dabei waren

	# Schulen Last 3 Years
	set SL3YNotTemp := {<i, j, k, l, m, n, o ,p ,q, r, s> in Beteiligte with n == 0 and m == 0};		# Ids aller Schulen, die in den letzten 2 Jahren nicht dabei waren
	set SL3YAppliedTemp := {<i, j, k, l, m, n, o ,p ,q, r, s> in Beteiligte with s > 0 and r > 0};		# Ids aller Schulen, die sich in allen letzten 2 Jahren angemeldet haben
	set SL3YNot := proj(SL3YNotTemp, <1>); 
	set SL3YApplied := proj(SL3YAppliedTemp, <1>); 
	set SL3Y := Schools inter SL3YNot inter SL3YApplied;								# IDs aller Schulen, die nun im dritten Jahr genommen werden sollen (sind angemeldet weil in S)
	

    # Bezirksliste (extern)
	set BezirkeBerlin := {read "input/regions.txt" as "<2s>"};	# Auflistung aller Berliner Bezirke
	set BezirkeUmland := Regions - BezirkeBerlin;						# Auflistung aller "Bezirke" aus dem berliner Umland
	set UmlS := {<i> in AppliedClasses with (RegionOfClass[i] == "___") or (RegionOfClass[i] == "Potsdam-Mittelmark")};

    # MultipleClasses-Termine
	set MultClassDate := {read "input/exceptions.txt" as "<2n>" match "^t"}; # Liste der MultipleClasses-Termin-IDs
	
    # Konflikte (extern)
	set Conflicts := {read "input/conflicts.txt" as "<2n, 3n, 4n>" match "^p"};	# Konflikte: Termin-Id, Klassen-Id, Klassen-Id
    # Vorgaben (extern)
	set ForceMatch := {read "input/doMatch.txt" as "<2n>" match "^b"};	# Vorgaben, das bestimmte Klassen einen Platz kriegen sollen: Klassen-Ids
    # Zuweisungen (extern)
	set ForceAssigment := {read "input/static.txt" as "<2n,3n>" match "^z"};# Zuweisungen von bestimmten Klassen zu bestimmten Kursen: Termin-Id, Klassen-Id
    #Kapazitaetserhoehungen (extern)
	set KapErh := {read "input/capIncrease.txt" as "<2n>" match "^t"};  #durchgefuehrte Kapazitaetserhoehungen beachten
	param bplus[KapErh] := read "input/capIncrease.txt" as "<2n> 3n" match "^t"; 

    # Datencheck  #wird in ausgabe.txt (oder konsole) geprintet
    do print "Checking data";
    do print "Anmeldungen |A|: ", card(Applications);
    do print "Termine |J|: ", card(Dates);
    do print "angemeldete Termine: ", card(proj(Applications,<2>));
    do print "unangemeldete Termine: ", card(Dates) - card(proj(Applications,<2>));
    do print "angemeldete Klassen |I|: ", card(AppliedClasses);
    do print "aufgelistete Klassen |O|: ", card(ClassIDs);
    do print "aufgelistete Schulen |S|: ", card(Schools);
    do print "Schulen, die in den letzten 3 Jahren angemeldet aber nicht genommen wurden |SL3Y|: ", card(SL3Y);
    do print "aufgelistete Bezirke |B|: ", card(Regions);
    do print "MultipleClasses-Termine |P|: ", card(MultClassDate);
    do print "Konflikte |C|: ", card(Conflicts);
    do print "Vorgaben |V|: ", card(ForceMatch);
    do print "Zuweisungen |Z|: ", card(ForceAssigment);

    set Nul := {0};

	do forall <i,j>   in Applications do check <i> in ClassIDs;
	do forall <i,j>   in Applications do check <j> in Dates;
	do forall <i,j>   in Applications do check a[i,j] >= 0;
	do forall <i,j>   in Applications with ClassGrade[i] < cmin[j] do print "Klasse ", i, ": Stufe ", ClassGrade[i], " < Termin ", j, ": min. Stufe ", cmin[j];
	do forall <i,j>   in Applications with ClassGrade[i] > cmax[j] do print "Klasse ", i, ": Stufe ", ClassGrade[i], " > Termin ", j, ": max. Stufe ", cmin[j];

	do forall <j>     in Dates do check cmin[j] <= cmax[j];

	do forall <j>     in MultClassDate do check <j> in Dates; 				
#	do forall <j>     in MultClassDate with bP[j] != b[j] do print "Termin ", j, ": Laborkapaziaet ", bP[j], " != Terminkapazitaet ", b[j];
 
#	do forall <i,j,k> in Conflicts do check <j,i> in Applications;
#	do forall <i,j,k> in Conflicts do check <k,i> in Applications;

#	do forall <i>     in ForceMatch do check <i> in AppliedClasses;
#	do forall <i,j>   in ForceAssigment do check <i,j> in Applications;
	
	var x[Applications] integer >= 0 <= 1;				        # Variable der Anmeldungen 
	var y[Schools] integer >= 0 <= 1;								    # Variable der Zuordnung der Schulen
	var z[Regions] integer >= 0 <= 1;								    # Variable der Zuordnung der Bezirke
	var w[Dates] integer >= 0 <= 1;								    # Variable der Zuordnung der Termine
	var yneu integer >= 0;                                      # Variable der Anzahl neuer Schulen
	var xsum integer >= 0;                                      # Variable der Anzahl zugewiesener Schueler
	var ysum integer >= 0;                                      # Variable der Anzahl zugewiesener Schulen
	var ksum integer >= 0;                                      # Variable der Anzahl zugewiesener Klassen
	var jsum integer >= 0;                                      # Variable der Anzahl zugewiesener Termine
	var yL3Y integer >= 0;                                      # Variable der Anzahl Schulen, die sich die letzten 2 Jahre vergeblich angemeldet haben und nun zugewiesen wurden
	var yL3Ytotal integer >= 0;				    # Variable der Anzahl Schulen, die sich die letzten 2 Jahre vergeblich angemeldet haben 
	var yL3YNTTT[SL3Y] integer >=0 <= 1; 			    # yL3YNotTakenThisTime: yL3YNTTT[s]=1 Wenn s in SL3Y aber dieses Jahr nicht genommen, 0 sonst
	%s                                                          # String7: Weitere Variablen
    

	#IP
	#-----------------------------------
	maximize obj: 									# Zielfunktion
		  OBJX * xsum								# Maximiere Schueleranzahl
		+ OBJY * yneu								# Maximiere Anzahl neuer Schulen
		+ OBJZ * ysum							    	# Maximiere Anzahl Schulen
		+ OBJK * ksum								# Maximiere Anzahl Klassen
		+ OBJT * jsum								# Maximiere Anzahl Termine	
	#	+ sum <s> in SL3Y: OBJ3Y*y[s]						# Maximiere Anzahl Schulen, die die letzten 3 Jahre nicht dabei waren(aber angemeldet)
        %s;                                                     # String7b: Zusaetzlicher Zielfunktionsterm
	#	+ sum <i,j> in Applications with <i> in UmlS: x[i,j];              # Maximiere Anzahl Schulen im Umland
	#	+ sum <t>   in BezirkeUmland: OBJZ*2*z[t];				# Maximiere Anzahl Bezirke
		
	subto pack: forall <i> in AppliedClasses do								# Bedingung einer Zuordnung
		sum <i,j> in Applications : x[i,j] <= 1;
	

	subto cap: %s                                               # Kapazitaet Termin j, String8: $forallJ
		sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j];


#	subto pack2: if (card(MultClassDate) > 0) then forall <p> in MultClassDate do	    # Bedingung der SingleClass-Termine !!ALT!!
#		sum <i,p> in Applications : x[i,p] <= 1 else 0 == 0 end;

	subto pack2: forall <j> in Dates without MultClassDate do	    			# Terminen darf grundsätzlich nur eine Klasse (bzw Anmeldung) zugewiesen werden, außer sie erlauben mehrere !!NEU!!
		sum <i,j> in Applications : x[i,j] <= 1;

	subto min_grade: forall <j> in Dates do                         # min. Alterstufe
        sum <i,j> in Applications with ClassGrade[i] < cmin[j]: x[i,j] == 0;

	subto max_grade: forall <j> in Dates do                         # max. Alterstufe
        sum <i,j> in Applications with ClassGrade[i] > cmax[j]: x[i,j] == 0;

#	subto age: forall <j> in Dates  without MultClassDate do                     # Alterskonflikt !!ALT!!
#		forall <i1> in ClassIDs with <i1,j> in Applications do
#			forall <i2> in ClassIDs with <i2,j> in Applications and ((ClassGrade[i1] - ClassGrade[i2] >= AGE) or (ClassGrade[i2] - ClassGrade[i1] >= AGE)) do
#				x[i1,j] + x[i2,j] <= 1;

	subto age: forall <p> in MultClassDate do                     			# Alterskonflikt !!NEU!!
		forall <i1> in ClassIDs with <i1,p> in Applications do
			forall <i2> in ClassIDs with <i2,p> in Applications and ((ClassGrade[i1] - ClassGrade[i2] >= AGE) or (ClassGrade[i2] - ClassGrade[i1] >= AGE)) do
				x[i1,p] + x[i2,p] <= 1;

	subto terminwahl: forall <i,j> in Applications do						# Beruecksichtigung von verschiedenen Klassen
		x[i,j] <= w[j];
		
	subto terminwahl2: forall <j> in Dates do						# nicht zu viele zaehlen
		sum <i,j> in Applications : x[i,j] >= w[j];
		
    subto count_termine:                                        # zugewiesene Termine zaehlen
        sum <j> in Dates: w[j] == jsum;

    subto count_classes:                                        # zugewiesene Klassen zaehlen
        sum <i,j> in Applications: x[i,j] == ksum;

	subto school: forall <s> in Schools do							# bei mehreren Klassen Schule nur einmal zaehlen
		sum <i,j> in Applications with s == SchoolOfClass[i]: x[i,j] >= y[s];
		
	subto neu2009:
		sum <s> in Snew: y[s] == yneu;

	subto yL3Y:												
		sum <s> in SL3Y: y[s] == yL3Y;

	subto yL3Ytotal: 											
		yL3Ytotal == card(SL3Y);
	subto sL3YNotTakenThisTime: forall <s> in SL3Y do							# Schulen die in SL3Y sind aber nicht genommen wurden
		yL3YNTTT[s] == 1-y[s];										# haben jetzt yL3YNTTT[s]=1

    subto count_pupils:                                         # Schueler zaehlen
        sum <i,j> in Applications: a[i,j]*x[i,j] == xsum;

    subto count_schools:                                        # Schulen zaehlen
        sum <s> in Schools: y[s] == ysum;

	subto quarter: forall <bez> in Regions do							# Beruecksichtigung von verschiedener Schulen
		sum <i,j> in Applications with bez == RegionOfClass[i]: x[i,j] >= z[bez];		# als mehrere aus einem Bezirk
		
#	subto konflikte: 									        # Konflikt-Ungleichungen (jetzt dynamisch?)
#		forall <i,j,k> in Conflicts do
#			x[j,i] + x[k,i] <= 1;

    # Vorgaben

	#subto pack_fix:										    # feste Klassen/Kurs-Zuordnung
	#	forall <i,j> in ForceAssigment do
	#		x[j,i] == 1;
	
	%s                                                          # String9: Besonderheiten je Modell

END_OF_TEMPLATE

#------------------------------------------------------------------------------
# Erstellung der Besonderheiten des LPs fuer das jeweilige Problem
#------------------------------------------------------------------------------

#..............................................................................
# Modell kip_main
#..............................................................................
# Mindestanzahl neuer Schulen

if($ARGV[5] eq "kip_main"){

    # Zielfunktion
    $lambda0 = $ARGV[0];
    $lambda1 = $ARGV[1];
    $lambda2 = $ARGV[2];
    $lambda3 = $ARGV[3];
    $lambda4 = $ARGV[4];

    $obj = "+ sum <s> in SL3Y: OBJ3Y*y[s]";					# : priorisiere Schulen,die die letzten 2 Jahre angemeldet aber nicht genommen
	# Kapazitaetsbedingung fuer alle Kurse
	$forallJ = "forall <j> in Dates without KapErh do";
	
	$constr = sprintf(
		"subto konflikte: if (card(Conflicts) > 0) then forall <i,j,k> in Conflicts do   # Konflikte
		    x[j,i] + x[k,i] <= 1 else 0 == 0 end;

	     subto zuweisung: if (card(ForceMatch) > 0) then forall <i> in ForceMatch do       # Vorgaben
			sum <i,j> in Applications : x[i,j] == 1 else 0 == 0 end;

	     subto pack_fix: if (card(ForceAssigment) > 0) then forall <i,j> in ForceAssigment do		 # feste Klassen/Kurs-Zuordnung
	        x[j,i] == 1 else 0 == 0 end;

	    subto cap_Erh: forall <j> in KapErh do    				# beachte KapErhoehung
		  sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j]+ bplus[j];
	    ");

	
	# Vorgaben, Konflikte, Zurodnungen, ggf. Mindestanzahl neuer/aller Schulen
	if($ARGV[6] ne ""){
		$constr .= sprintf(
            "
             subto schoolneu: 									         # neue Schulen
				sum <s> in Snew: y[s] >= $ARGV[6];
	        ");
	}

	if($ARGV[7] ne ""){
		$constr .= sprintf(
            "
             subto schoolall: 									         # Schulen
				sum <s> in S: y[s] >= $ARGV[7];
	        ");
	}
	
	if($ARGV[8] ne ""){
		$constr .= sprintf(
            "
             subto minSchueler: 								         # Schueler
				xsum >= $ARGV[8];
	        ");
	}
	
	if($ARGV[9] ne ""){
		$constr .= sprintf(
            "
             subto minKlassen: 									         # Klassen
				ksum >= $ARGV[9];
	        ");
	}
	
	if($ARGV[10] ne ""){
		$constr .= sprintf(
            "
             subto minTermine: 									         # Termine
				jsum >= $ARGV[10];
	        ");
	}
}

#..............................................................................
# Modell kip_gapAnalyzation
#..............................................................................

elsif($ARGV[5] eq "kip_gapAnalyzation"){

    # Zielfunktion
    $lambda0 = 0;
    $lambda1 = 0;
    $lambda2 = 0;
    $lambda3 = 0;
    $lambda4 = 0;

	# Maximierung der Ueberdeckung eines einzigen Termins
	$param = "set F := { $ARGV[6] };";
      
	$obj = "+ sum <i,j> in Applications with <j> in F: a[i,j]*x[i,j]";  #original
	#$obj = "+ sum <i,j> in Applications with <j> in F: a[i,j]*x[i,j] + sum <i,j> in Applications with <j> in F and <SchoolOfClass[i]> in SL3Y: OBJ3Y*a[i,j]*x[i,j] "; 
	$forallJ = "forall <j> in Dates without KapErh do";

	$constr = sprintf(
		"subto konflikte: if (card(Conflicts) > 0) then forall <i,j,k> in Conflicts do   # Konflikte
		    x[j,i] + x[k,i] <= 1 else 0 == 0 end;

	     subto zuweisung: if (card(ForceMatch) > 0) then forall <i> in ForceMatch do       # Vorgaben
			sum <i,j> in Applications : x[i,j] == 1 else 0 == 0 end;

	     subto pack_fix: if (card(ForceAssigment) > 0) then forall <i,j> in ForceAssigment do		 # feste Klassen/Kurs-Zuordnung
	        x[j,i] == 1 else 0 == 0 end;
	   
	      subto cap_Erh: forall <j> in KapErh do    				# beachte KapErhoehung
		 sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j]+ bplus[j];
	    


	    ");
#	    
#	    subto cap_Erh: forall <j> in F inter KapErh do
#		 sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j]+ bplus[j];

	# Betrachtung einzelner Termine
#	$param = "param j := $ARGV[6];";
}

#..............................................................................
# Modell kip_eConstraint
#..............................................................................

elsif($ARGV[5] eq "kip_eConstraint"){

    # Zielfunktion
    $lambda0 = $ARGV[0];
    $lambda1 = $ARGV[1];
    $lambda2 = $ARGV[2];
    $lambda3 = $ARGV[3];
    $lambda4 = $ARGV[4];

    $obj = "+ sum <s> in SL3Y: OBJ3Y*y[s]";					# : priorisiere Schulen,die die letzten 2 Jahre angemeldet aber nicht genommen

	$param = sprintf(
			"param lb := $ARGV[6];                               # Anzahl neue Schulen
#			 param ue := 0;
            ");
	# variable Kap.erhÃ¶hung je Termin
#	$var = "var u[Dates] >=0;";

	# Kapazitaetsbedingung fuer alle Kurse
	$forallJ = "forall <j> in Dates without KapErh do";

	# Zuweisung bestimmter Klassen und
	# u=Pauschaler Wert fÃ¼r alle Termine  J
	# Anzahl neuer Schulen =parameter l=L

	$constr = sprintf(
		   "subto konflikte: if (card(Conflicts) > 0) then forall <i,j,k> in Conflicts do   # Konflikte
		        x[j,i] + x[k,i] <= 1 else 0 == 0 end;

	        subto zuweisung: if (card(ForceMatch) > 0) then forall <i> in ForceMatch do       # Vorgaben
			    sum <i,j> in Applications : x[i,j] == 1 else 0 == 0 end;

	        subto pack_fix: if (card(ForceAssigment) > 0) then forall <i,j> in ForceAssigment do		 # feste Klassen/Kurs-Zuordnung
	            x[j,i] == 1 else 0 == 0 end;

			subto schoolneu:
				yneu == lb;

	    subto cap_Erh: forall <j> in KapErh do    				# beachte KapErhoehung
		  sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j]+ bplus[j];
            ");

#			subto total_flexkap:
#				sum <j> in Dates : u[j] <=ue;

#				ysum == lb;     # maxSchulenMaxSchueler, maxSchulenMaxSchulen ändern

}

#..............................................................................
# Modell kip_eConstraint2
#..............................................................................

elsif($ARGV[5] eq "kip_eConstraint2"){

    # Zielfunktion
    $lambda0 = $ARGV[0];
    $lambda1 = $ARGV[1];
    $lambda2 = $ARGV[2];
    $lambda3 = $ARGV[3];
    $lambda4 = $ARGV[4];

    $obj = "+ sum <s> in SL3Y: OBJ3Y*y[s]";					# : priorisiere Schulen,die die letzten 2 Jahre angemeldet aber nicht genommen

	$param = sprintf(
			"param lb := $ARGV[6];                               # Anzahl neue Schulen
#			 param ue := 0;
            ");
	# variable Kap.erhÃ¶hung je Termin
#	$var = "var u[Dates] >=0;";

	# Kapazitaetsbedingung fuer alle Kurse
	$forallJ = "forall <j> in Dates without KapErh do";

	# Zuweisung bestimmter Klassen und
	# u=Pauschaler Wert fÃ¼r alle Termine  J
	# Anzahl neuer Schulen =parameter l=L

	$constr = sprintf(
		   "subto konflikte: if (card(Conflicts) > 0) then forall <i,j,k> in Conflicts do   # Konflikte
		        x[j,i] + x[k,i] <= 1 else 0 == 0 end;

	        subto zuweisung: if (card(ForceMatch) > 0) then forall <i> in ForceMatch do       # Vorgaben
			    sum <i,j> in Applications : x[i,j] == 1 else 0 == 0 end;

	        subto pack_fix: if (card(ForceAssigment) > 0) then forall <i,j> in ForceAssigment do		 # feste Klassen/Kurs-Zuordnung
	            x[j,i] == 1 else 0 == 0 end;

			subto school_all:
				ysum == lb;

	    subto cap_Erh: forall <j> in KapErh do    				# beachte KapErhoehung
		  sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j]+ bplus[j];
            ");

}

#..............................................................................
# Modell kip_capacityIncrement
#..............................................................................

elsif($ARGV[5] eq "kip_capacityIncrement"){

    # Zielfunktion
    $lambda0 = $ARGV[0];
    $lambda1 = $ARGV[1];
    $lambda2 = $ARGV[2];
    $lambda3 = $ARGV[3];
    $lambda4 = $ARGV[4];

  $obj = "+ sum <s> in SL3Y: OBJ3Y*y[s]";					# : priorisiere Schulen,die die letzten 2 Jahre angemeldet aber nicht genommen

	#Parameter fÃ¼r die KapazitÃ¤tserhoehung und zu analysierenden Termin
	$param = sprintf(
			"param e := $ARGV[6]; 
			param rr := $ARGV[7];
			set F := {rr};");
	$forallJ = "forall <j> in Dates without KapErh without F do";


	#HIER: KapazitÃ¤tserhÃ¶hung des Termins rr
	if($ARGV[8] eq ""){
		$constr = sprintf(
		   "subto konflikte: if (card(Conflicts) > 0) then forall <i,j,k> in Conflicts do   # Konflikte
		        x[j,i] + x[k,i] <= 1 else 0 == 0 end;

	        subto zuweisung: if (card(ForceMatch) > 0) then forall <i> in ForceMatch do       # Vorgaben
			    sum <i,j> in Applications : x[i,j] == 1 else 0 == 0 end;

	        subto pack_fix: if (card(ForceAssigment) > 0) then forall <i,j> in ForceAssigment do		# feste Klassen/Kurs-Zuordnung
	            x[j,i] == 1 else 0 == 0 end;

			subto term_cap: if <rr> in KapErh then				# beachte auch hier evtl KapErhoehung!	
				sum <i,rr> in Applications : a[i,rr] * x[i,rr] <= b[rr]+e+bplus[rr]
				else sum <i,rr> in Applications : a[i,rr] * x[i,rr] <= b[rr]+e end;

	    subto cap_Erh: forall <j> in KapErh without F do    				# beachte KapErhoehung
		  sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j]+ bplus[j];");
	}
	else{
		$constr = sprintf(
		   "subto konflikte: if (card(Conflicts) > 0) then forall <i,j,k> in Conflicts do   # Konflikte
		        x[j,i] + x[k,i] <= 1 else 0 == 0 end;

	        subto zuweisung: if (card(ForceMatch) > 0) then forall <i> in ForceMatch do       # Vorgaben
			    sum <i,j> in Applications : x[i,j] == 1 else 0 == 0 end;

	        subto pack_fix: if (card(ForceAssigment) > 0) then forall <i,j> in ForceAssigment do		# feste Klassen/Kurs-Zuordnung
	            x[j,i] == 1 else 0 == 0 end;

			subto term_cap: if <rr> in KapErh then				# beachte auch hier evtl KapErhoehung!	
				sum <i,rr> in Applications : a[i,rr] * x[i,rr] <= b[rr]+e+bplus[rr]
				else sum <i,rr> in Applications : a[i,rr] * x[i,rr] <= b[rr]+e end;

			subto schoolneu: 									
				sum <s> in Snew do y[s] >= $ARGV[8];

	    subto cap_Erh: forall <j> in KapErh without F do    				# beachte KapErhoehung
		  sum <i,j> in Applications : a[i,j]*x[i,j] <= b[j]+ bplus[j];");

	}
}




#..............................................................................
# kein Modell
#..............................................................................

else {
  die "$0: No KIP model chosen";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Erzeugung des zimpl-Modells
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

$target = "output/zimpl/$ARGV[5].zpl";

$zimplmodel = sprintf($TEMPLATE, $lambda0, $lambda1, $lambda2, $lambda3, $lambda4, $param, $var, $obj, $forallJ, $constr);

open(ZPLFILE, ">$target") or die "$0: Can't open file $target: $!\n";
print ZPLFILE $zimplmodel;
close(ZPLFILE) or die "$0: Can't close file $target: $!\n";
